import { useRef, useCallback, useContext, useEffect, useState } from "react";
import Container from "./Container";
import ColumnGrid from "./ColumnGrid";
import html2canvas from "html2canvas";

import ImagesContext from "../store/images-context";
import { toast } from "react-toastify";

const Grid = ({ classes, layout, layoutFormat }) => {
	const { cols } = layout;
	const ref = useRef();
	const [title, setTitle] = useState("");

	const [isSaved, setIsSaved] = useState(false);
	const imageCTX = useContext(ImagesContext);
	const attachmentIds = imageCTX.images.map(image => image.attachmentId);

	let cols_css = { gridTemplateColumns: "" };

	if (cols.length > 0) {
		if (cols.length > 1) {
			cols.forEach((item) => {
				cols_css.gridTemplateColumns += ` calc(${item.width}% - 3px)`;
			});
		} else if (cols.length === 1) {
			cols.forEach((item) => {
				cols_css.gridTemplateColumns += ` ${item.width}%`;
			});
		}
	}

	const onReset = () => {
		window.location.reload(false);
	};

	const onButtonClick = useCallback(() => {
		if (ref.current === null) {
			return;
		}

		const id = toast.loading('Saving new image...');

		html2canvas(ref.current, { allowTaint: true, useCORS: true }).then((canvas) => {
			const imageData = canvas.toDataURL().replace(/.*,/, '');

			/*eslint-disable no-undef*/
			fetch(ajaxurl + '?action=dmg_upload_split_image', {
				method: 'POST',
				headers: {
					'Content-Type': 'application/json',
				},
				referrerPolicy: "no-referrer",
				body: JSON.stringify({
					imageData: imageData,
					title: title,
					attachmentIds: attachmentIds
				}),
			})
				.then((response) => response.json())
				.then((data) => {
					toast.update(id, {
						render: () => <a href={data.message.admin_url}>{data.message.info}</a>,
						type: "success",
						isLoading: false,
						autoClose: 5000,
					});
					setIsSaved(true);
				})
				.catch((error) => {
					toast.update(id, {
						render: "Unable to save image.",
						type: "error",
						isLoading: false,
						autoClose: 5000,
					});
				});

			/*eslint-enable no-undef*/
		});
	}, [ref, title, attachmentIds]);

	useEffect(() => { }, [imageCTX.images]);

	return (
		<>
			<div className={classes} style={cols_css} ref={ref}>
				{cols.map((item, key) => {
					if (Array.isArray(item.height)) {
						return (
							<div className="inner-grid" key={item.id}>
								<ColumnGrid
									layout={item.height}
									layoutFormat={layoutFormat}
								/>
							</div>
						);
					}

					const style = {
						height: `${layoutFormat.height}px`,
						backgroundColor: "#C0C3CA",
					};
					return (
						<Container
							styles={style}
							key={item.id}
							classes=''
							item={item}
						/>
					);
				})}
			</div>
			<div style={{ marginTop: "10px" }}>
				<input
					type="text"
					class="widefat"
					name="image-composition-title"
					value={title}
					placeholder="Image title (optional)"
					onChange={(e) => setTitle(e.target.value)}
				/>
				<button
					className="button button-primary button-large"
					style={{ marginTop: "10px" }}
					onClick={onButtonClick}
				>
					Save Image
				</button>
				{isSaved ? (
					<button
						className="button button-large"
						style={{ marginTop: "10px", marginLeft: "10px" }}
						onClick={onReset}
					>
						Reset
					</button>
				) : (
					<></>
				)}
			</div>
		</>
	);
};

export default Grid;
