<?php
/**
 * Plugin Name:     DMG Image Compositions
 * Plugin URI:      https://github.com/ANMedia/dmg-image-compositions-wp-plugin
 * Description:     Create Image Split Compositions in WordPress
 * Author:          DMG
 * Text Domain:     image-compositions
 * Version:         2.0.1
 *
 * @package         \DMG\Image_Compositions
 */

namespace DMG;

/**
 * Class \DMG\Image_Compositions
 *
 * Create Image Compositions in WordPress
 *
 * @package DMG
 */
class Image_Compositions {

	/**
	 * Menu slug
	 *
	 * @var string
	 */
	const MENU_SLUG = 'dmg-image-compositions';

	/**
	 * Initialize
	 */
	public static function init() {
		add_action( 'admin_menu', [ __CLASS__, 'add_submenu_page' ] );
		add_action( 'admin_enqueue_scripts', [ __CLASS__, 'enqueue_scripts' ] );
		add_action( 'wp_ajax_dmg_upload_split_image' , [ __CLASS__, 'upload_image_composition' ]);
	}

	 /**
	 * Add submenu page
	 *
	 * @return void
     */
	public static function add_submenu_page() {
		add_submenu_page( 
			'upload.php',
			'Image Compositions',
			'Image Compositions',
    		'upload_files',
    		self::MENU_SLUG,
    		[ __CLASS__, 'page_callback' ]
    	);
	}

	 /**
	  * The page callback
	  *
	  * @return void
	  */
	public static function page_callback() {
		?>
		<div class="wrap">
			<h2>Image Compositions</h2>
			<div id="root"></div>
		</div>
		<?php
	}

	 /**
	  * Add assets
	  *
	  * @return void
	  */
	public static function enqueue_scripts() {

		// Ensure we're on the correct page
		global $pagenow;
		if ( $pagenow !== 'upload.php' || empty( $_GET['page'] ) || $_GET['page'] !== self::MENU_SLUG ) {
			return;
		}

		// Enqueue standard media scripts
		wp_enqueue_media();

		$assets_json = file_get_contents( __DIR__ . '/assets/build/asset-manifest.json' );
		$assets = json_decode($assets_json, true);

		if( isset( $assets['files']['main.js'] ) ) {
			wp_enqueue_script(
				self::MENU_SLUG,
				plugin_dir_url( __FILE__ ) . 'assets/build' .$assets['files']['main.js'],
				[],
				null, 
				true
			);
		}

		if( isset($assets['files']['main.css'] ) ) {
			wp_enqueue_style(
				self::MENU_SLUG,
				plugin_dir_url( __FILE__ ) .  'assets/build' . $assets['files']['main.css']
			);
		}		
	}

	 /**
	  * Process new image
	  *
	  * @return void
	  */
	public static function upload_image_composition() {

		$entityBody = file_get_contents( 'php://input' );

		$postData = json_decode( $entityBody, true );

		// Check for attachment IDs
		if ( empty( $postData['attachmentIds'] )) {
			echo json_encode([
				"message" => "Unable to save image. No attachment IDs passed.",
				"status" => "error"
			]);
			exit();
		}

		$attachment_ids = array_map( 'intval', $postData['attachmentIds'] );

		// Make sure data is set
		if ( empty( $postData['imageData'] )) {
			echo json_encode([
				"message" => "Unable to save image. No Composition data.",
				"status" => "error"
			]);
			exit();
		}

		// Extract image data
		$data  = base64_decode( preg_replace('#^data:image/\w+;base64,#i', '', $postData['imageData']) );
		$image = imagecreatefromstring( $data );

		// Require files that may not have been included
		require_once( ABSPATH . 'wp-admin/includes/file.php' );
		require_once( ABSPATH . 'wp-admin/includes/media.php' );
		require_once( ABSPATH . 'wp-admin/includes/image.php' );

		// Sanitize image title if passed
		$image_title = ( ! empty( $postData['title'] ) ) ? sanitize_text_field( $postData['title'] ) : '';

		// Determine file names
		$filename = ( ! empty( $image_title ) ) ? sanitize_title( $image_title ) : 'comp-' . time();
		$filename .= '.png';
		$tempnam  = wp_tempnam( $filename );

		imagepng( $image, $tempnam );

		// Save temp file
		$files = [
			'name'     => $filename,
			'tmp_name' => $tempnam,
		];

		// Pass image title if set
		$post_data = [];
		if ( ! empty( $image_title ) ) {
			$post_data['post_title'] = $image_title;
		}

		// Upload the new image
		$uploaded_id = media_handle_sideload(
			$files,
			0,
			null,
			$post_data
		);

		// Redirect to upload.php
		if ( $uploaded_id > 0 ) {

			/**
			 * Run an action when composition successfully uploaded.
			 *
			 * @param int $uploaded_id The post ID of the uploaded attachment.
			 */
			do_action( 'dmg_image_composition_uploaded', $uploaded_id );

			// Add post meta for each source image ID
			foreach ( $attachment_ids as $image_id ) {
				add_post_meta( $uploaded_id, 'dmg_image_composition_source_id', $image_id );
			}

			echo json_encode([
				'message' => array(
					'info'				=> $filename . ' has beed created.',
					'admin_url'			=> '/wp-admin/upload.php?item=' . $uploaded_id,
					'attachement_url' 	=> wp_get_attachment_url( $uploaded_id )
				),

				"status" => "success"
			]);
			exit();
			
		}

		echo json_encode([
			"message" => "Unable add image.",
			"status" => "error"
		]);
		
		exit();
	}
}

Image_Compositions::init();
